include ".\eqoutputtab.prg"

!debug = 0

if !debug = 0 then
	logmode +addin
endif

!dogui = 1
%bootstraps = "10000"
!domean=0
!dose=1
!dograph=1
!bstype = 1   'type of bootstrap method.  1=resample variables,  2=generate residuals

'check that an object exists
%type = @getthistype
if %type="NONE" then
	@uiprompt("No object found, please open an Equation object")
	stop
endif

'check that _this object is an equation
if %type<>"EQUATION" then
	@uiprompt("Procedure can only be run from an Equation object")
	stop
endif

'check that equation was estimated by LS, and by list.
%method = _this.@method
if %method <> "LS" then
	@uiprompt("Procedure can only be run from Equations estimated by Least Squares")
	stop
endif
if _this.@bylist = 0 then
	@uiprompt("Procedure can only be run from Equations estimated by list")
	stop
endif

'parse options
if @len(@option(1)) > 0 then
	%bootstraps = @equaloption("BS")
	!domean = @hasoption("M")
	!dose = @hasoption("SE")
	!dograph = @hasoption("G")
	!bstype = @hasoption("RES") + 1
	!dogui = 0
endif


%types = """Resample variables"" ""Resample residuals"""
'dialog
if !dogui = 1 then
	!result = @uidialog("list", !bstype, "Type of bootstrapping", %types, "edit", %bootstraps, "Enter a number of bootstrap repetitions", "check", !domean, "Bootstrap coefficient point estimates", "Check", !dose, "Bootstrap coefficient standard errors", "check", !dograph, "Show bootstrap distribution", "caption", "Bootstrap")
	if !result = -1 then
		stop
	endif
endif


!bootstraps = @val(%bootstraps)
!k = _this.@ncoef
!obs = _this.@regobs
%os = @pagesmpl
%options = _this.@options
%names = _this.@varlist
%dep = @word(%names,1)
%dnames = @wmid(%names,2) 'drop dependent

'make residuals, to grab equation's estimation sample
%res = @getnextname("_res")
_this.makeresid {%res}
smpl if {%res} <> na

'create matrix to store coefs
%c = @getnextname("_coefs")
matrix(!bootstraps,!k) {%c}

'name of equation
%eq = @getnextname("_es")

if !bstype = 1 then
	'make group of regressors
	%r = @getnextname("_regs")
	group {%r} {%names}
	
	'set of names for bootstrapped series
	%newnames = ""
	for !i = 1 to !k+1
		%name = {%r}.@seriesname(!i)
		%newname = @getnextname(%name)
		%newname = @makevalidname(%newname)
		%newnames = %newnames + " " + %newname
	next
	
	'name of group containing strapped
	%bs = @getnextname("_bs")
	
	'bootstrap
	!pct = 5
	!status = !bootstraps*!pct/100
	%msg = "Performing bootstrap repetitions"
	statusline {%msg}
	for !i=1 to !bootstraps
		{%r}.resample(name={%bs}) {%newnames}
		equation {%eq}.ls({%options}) {%bs}
		rowplace({%c},@transpose({%eq}.@coefs),!i)
		d {%newnames}	
		if !i>!status then
			%msg = %msg + "...." + @str(!pct) + "%"
			statusline {%msg}
			!pct = !pct + 5
			!status = !bootstraps*!pct/100
		endif
	next
else
	%resbs = @getnextname("_resbs")
	%yres = @getnextname("_yres")
	!pct = 5
	!status = !bootstraps*!pct/100
	%msg = "Performing bootstrap repetitions"	
	statusline {%msg}
	for !i = 1 to !bootstraps
		{%res}.resample {%resbs}
		series {%yres} = {%dep} + {%resbs}
		equation {%eq}.ls({%options}) {%yres} {%dnames}	
		rowplace({%c},@transpose({%eq}.@coefs),!i)
		d {%yres} {%resbs}	
		if !i>!status then
			%msg = %msg + "...." + @str(!pct) + "%"
			statusline {%msg}
			!pct = !pct + 5
			!status = !bootstraps*!pct/100
		endif		
	next
endif


'calculate SEs
%cov = @getnextname("_cov")
sym {%cov} = @cov({%c})

'calculate means
%means = @getnextname("_means")
if !domean=1 then
	vector {%means} = @cmean({%c})
else
	vector {%means} = _this.@coefs
endif

'make results table
%tn = @getnextname("_results")
table(10,5) {%tn}
%title = "Dependent variable: " + %dep
%method= "Least Squares"
%sstr = _this.@smpl
%obs = @str(_this.@regobs)
!row = 1
call maketable({%tn}, !row, {%means}, {%cov}, %dnames, %title, %method, %sstr, %obs, _this.@df, "")

'insert row containing SE information
{%tn}.insertrow(6)
%tempstr = "Bootstrapped " 
if !domean = 1 then
	%tempstr = %tempstr + "coefficient estimates "
	if !dose = 1 then
		%tempstr = %tempstr + "and "
	endif
endif
if !dose=1 then
	%tempstr = %tempstr + "standard errors "
endif
%tempstr = %tempstr + "(" + %bootstraps + " repetitions)"
{%tn}(6,1) = %tempstr

'add footer information

'first find location of the footer in existing table, by searching for "R-squared" text
%tn2 = @getnextname("_temp")
freeze({%tn2}) _this.results
!i=1
while !i>0
	if {%tn2}(!i,1) = "R-squared" then
		exitloop
	endif
	!i=!i+1
wend
!startrow = !i-1
!endrow = !i+10
{%tn2}.copyrange !startrow 1 !endrow 5 {%tn} !row 1


if !dograph=1 then
	%gn = @getnextname("_graph")
	%vtemp = @getnextname("_vtemp")
	%gnames = ""
	for !i=1 to !k
		%gtemp = @getnextname("_gtemp")
		vector {%vtemp} = @columnextract({%c},!i)
		freeze({%gtemp}) {%vtemp}.distplot freqpoly
		%name = @word(%dnames,!i)
		{%gtemp}.addtext(t) {%name}
		%gnames = %gnames + " " + %gtemp
		d {%vtemp}
	next
	if !k > 1 then
		graph {%gn}.merge {%gnames}
	else
		%gn = %gnames
	endif
	%spname = @getnextname("_spool")
	spool {%spname}
	{%spname}.append {%tn}
	{%spname}.append {%gn}
	{%spname}.flatten
	{%spname}.options -tree -titles -margins -borders
	{%spname}.graphmode(type=fixed) 12
	_this.display {%spname}
else 
	_this.display {%tn} 
endif


d(noerr) {%r} {%bs} {%eq} {%res} {%tn2} {%tn} {%means} {%cov} {%c} {%gnames} {%spname}
if @isobject(%gn) then
	d {%gn}
endif
smpl {%os}
statusline
